;;;
;;; The evaluator for formulas of the propositional calculus.
;;; Determines the truth value of a formula, given an environment
;;; which lists the true propositions.
;;;
evalformula[formula; env] =
  [proposition-p[formula] -->
     [defined-p[env; prop-name[formula]] -->
        apply-env[env; prop-name[formula]];
      otherwise --> error[(Undefined proposition)]];
   negation-p[formula] -->
     [evalformula[negend[formula]; env] --> F;
      otherwise --> T];
   conjunction-p[formula] -->
     [evalformula[conjunct1[formula]; env] -->
        evalformula[conjunct2[formula]; env];
      otherwise --> F];
   disjunction-p[formula] -->
     [evalformula[disjunct1[formula]; env] --> T;
      otherwise --> evalformula[disjunct2[formula]; env]];
   implication-p[formula] -->
     [evalformula[antecedent[formula]; env] -->
        evalformula[consequent[formula]; env];
      otherwise --> T];
   equivalence-p[formula] -->
     [evalformula[condition1[formula]; env] -->
        evalformula[condition2[formula]; env];
      evalformula[condition2[formula]; env] --> F;
      otherwise --> T]]
;;;
;;; The following data access functions implement the environment ADT
;;; for propositions, using a list of all the true propositions.  All
;;; others are assumed to be false.
;;;
apply-env[env; name] =
  [endp[env] --> F;
   eq[first[env]; name] --> T;
   otherwise --> apply-env[rest[env]; name]]
;;;
;;; The following functions are not used in evalformula; they are
;;; provided for the completeness of the ADT implementation.
;;;
defined-p[env; name] = T

extend-env[env; name; value] =
  [value --> cons[name; env];
   otherwise --> remove-name[env; name]]

remove-name[env; name] =
  [endp[env] --> ();
   eq[first[env]; name] --> remove-name[rest[env]; name];
   otherwise --> cons[first[env]; remove-name[rest[env]; name]]]
